#!/bin/bash

PROJ_ROOT=$(realpath "$(
    cd "$(dirname "$0")" >/dev/null 2>&1
    pwd -P
)"/..)

# Version of program could be set by a git tag.
export VERSION=$(git describe --tags)
export GIT_COMMIT=$(git rev-list -1 HEAD --abbrev-commit)
export BUILD_TIME="$(date +%FT%T)"
export GO_OS_ARCH="$(go env GOOS)/$(go env GOARCH)"
export API_VERSION=$(cat $PROJ_ROOT/api.version)

echo "Version: $VERSION"
echo "GIT_COMMIT: $GIT_COMMIT"
echo "BUILD_TIME: $BUILD_TIME"
echo "GO_OS_ARCH: $GO_OS_ARCH"
echo "API_VERSION: $API_VERSION"

export LD_FLAGS="-X git.skogstorpet.nu/staffan/environ.GitCommit=$GIT_COMMIT \
    -X git.skogstorpet.nu/staffan/environ.BuildTime=$BUILD_TIME \
    -X git.skogstorpet.nu/staffan/environ.GoOsArch=$GO_OS_ARCH \
    -X git.skogstorpet.nu/staffan/environ.Version=$VERSION \
    -X git.skogstorpet.nu/staffan/environ.APIVersion=$API_VERSION"

echo "BUILDING $PROJ_ROOT/.examples/server"
go build -ldflags "$LD_FLAGS" -o $PROJ_ROOT/.examples/server $PROJ_ROOT/example/server/main.go

echo "BUILDING $PROJ_ROOT/.examples/client"
go build -ldflags "$LD_FLAGS" -o $PROJ_ROOT/.examples/client $PROJ_ROOT/example/client/main.go

echo "Starting $PROJ_ROOT/.examples/server"
${PROJ_ROOT}/.examples/server &
spid=$!

echo "Starting $PROJ_ROOT/.examples/client"
$PROJ_ROOT/.examples/client &
sleep 2
echo Killing server
kill $spid
